local L = AceLibrary("AceLocale-2.2"):new("GlowFoSho")
local dewdrop = AceLibrary("Dewdrop-2.0")
local currWeaponLink, currWeaponEnchant
local tbl = {}
local showGlowless = false

--defined at the bottom
local enchants

GlowFoSho = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0","AceHook-2.1")

--[[

Local helper functions

--]]

--returns information about the enchant when provided with enchantID
local function enchantInfo(enchantID)
	if tonumber(enchantID) then
		for name, enchant in pairs(enchants) do
			if enchant.enchantID == enchantID then
				return name, enchant.formulaID, enchant.glow, enchantID
			end
		end
		return
	end
end

--creates a link for an enchanting recipe with formulaID and name
local function GetRecipeLink(formulaID,name)
	local is2H = string.find(name," %(2H%)")
	if is2H then
		name = string.sub(name,1,is2H-1)
		is2H = "2H "
	else
		is2H = ""
	end
	
	return "|cffffd000|Henchant:" .. formulaID .. "|h[Enchant " .. is2H .. "Weapon - " .. name .. "]|h|r"
end

--sets enchant for item link to enchantID
local function SetEnchant(link, enchantID)
	local link1, link2 = string.match(link,"(.*item:%d+:)%d+(.*)")
	if link1 and link2 then
		return link1 .. enchantID .. link2
	end
	
	return
end

--returns the enchantID in the link
local function GetEnchant(link)
	return string.match(link,"item:%d+:(%d+)")
end

--returns the list of enchants for dewdrop menu
local function GetEnchantList()
	for name, enchant in pairs(enchants) do
		if enchant.glow or showGlowless then
			tbl[name] = name
		else
			tbl[name] = nil
		end
	end
	return tbl
end

--[[

Main Addon Functions

--]]

function GlowFoSho:OnInitialize()
	self:RegisterChatCommand({"/glowfosho", "/gfs"})
	
	--set up GlowFoSho button on the dressup frame
	local gfsbutton = CreateFrame("Button","GlowFoSho_Button",DressUpFrame)
	--to be compatible with CloseUp, otherwise does not receive clicks
	gfsbutton:SetFrameStrata("HIGH")
	gfsbutton:SetWidth(25)
	gfsbutton:SetHeight(25)
	gfsbutton:SetNormalTexture("Interface\\Icons\\Ability_SearingArrow")
	gfsbutton:SetHighlightTexture("Interface\\Buttons\\ButtonHilight-Square","ADD")
	gfsbutton:SetPoint("TOPRIGHT","DressUpFrame","TOPRIGHT",-50,-80)
	
	--create the dropdown menu
	dewdrop:Register(GlowFoSho_Button,
					"point", function(parent) return "TOPLEFT", "TOPRIGHT" end,
					"children",function() dewdrop:FeedAceOptionsTable({
						type = "group",
						args = {
							getWeaponLink = {
								name = L["Show Weapon Link"],
								desc = L["Displays the link of the enchanted weapon in the chat frame."],
								type = "execute",
								func = "GetWeaponLink",
								handler = self,
								order = 10
							},
							getFormulaLink = {
								name = L["Show Enchant Link"],
								desc = L["Displays the link of the enchant currently on the weapon in the chat frame. CAUTION: links are not secure and may diconnect you!"],
								type = "execute",
								func = "GetEnchantLink",
								handler = self,
								order = 20
							},
							glowless = {
								name = L["Show Glowless Enchants"],
								desc = L["Allows you to preview enchants which do not add glow to a weapon."],
								type = "toggle",
								get = function() return showGlowless end,
								set = function(v) showGlowless = v end,
								order = 30
							},
							enchants = {
								name = L["Enchants"],
								desc = L["List of weapon enchants you can preview."],
								type = "text",
								get = function() return currWeaponEnchant end,
								set = "EnchantWeaponByName",
								validate = GetEnchantList(),
								handler = self,
								order = 40
							},
							close = {
								name = L["Close"],
								desc = L["Close the menu."],
								type = "execute",
								func = function() dewdrop:Close() end,
								order = 50
							},
						}
					}) end,
					"dontHook", true)
	--set up button click scripts
	gfsbutton:RegisterForClicks("LeftButtonUp","RightButtonUp")
	gfsbutton:SetScript("OnClick",function() if dewdrop:IsOpen(GlowFoSho_Button) then dewdrop:Close() else dewdrop:Open(GlowFoSho_Button) end end)

	--hook onto DressUp frame functions
	self:SecureHook("DressUpItemLink")
	self:HookScript(DressUpFrame,"OnShow","OnDressUpFrameShow")
	self:HookScript(DressUpFrameResetButton,"OnClick","OnDressUpFrameResetButtonClick")
end

--hook to DressUpItemLink function
function GlowFoSho:DressUpItemLink(link)
	if link then
		local iType
		_, link, _, _, _, iType = GetItemInfo(link)

		--if previewing weapon, set the link to passed link, otherwise set it to current weapon
		if iType == "Weapon" then
			currWeaponLink = link
		end
	end

	if not currWeaponLink then
		currWeaponLink = GetInventoryItemLink("player",GetInventorySlotInfo("MainHandSlot"))
	end

	currWeaponEnchant = enchantInfo(GetEnchant(currWeaponLink))
		
end

--reset previous enchants when DressUp frame shows up
function GlowFoSho:OnDressUpFrameShow()
	currWeaponLink = nil
	currWeaponEnchant = nil
	self.hooks[DressUpFrame]["OnShow"]()
end

--reset previous enchants when Reset button is pressed.
--enchant is set to currently equiped weapon data
function GlowFoSho:OnDressUpFrameResetButtonClick()
	currWeaponLink = GetInventoryItemLink("player",GetInventorySlotInfo("MainHandSlot"))
	currWeaponEnchant = enchantInfo(GetEnchant(currWeaponLink))
	self.hooks[DressUpFrameResetButton]["OnClick"]()
end

--enchant and preview weapon
-- args: enchant name as it appears in the enchants table
function GlowFoSho:EnchantWeaponByName(name)
	if enchants[name] then
		self:EnchantWeapon(enchants[name].enchantID)
		currWeaponEnchant = name
	end
end

--enchant and preview weapon by enchantID
function GlowFoSho:EnchantWeapon(enchantID)
	if currWeaponLink then
		local newLink = SetEnchant(currWeaponLink,enchantID)
		if newLink then
			currWeaponLink = newLink
			DressUpItemLink(currWeaponLink)
		end
	end
end

--prints out link to the currently previewed weapon with a currently previewed enchant
function GlowFoSho:GetWeaponLink()
	if currWeaponLink then
		self:Print(currWeaponLink)
	end
end

--prints out unsecure link to a formula for the current enchant on the weapon
function GlowFoSho:GetEnchantLink()
	local _, name, formulaID = self:GetCurrEnchant()
	if name then
		self:Print(GetRecipeLink(formulaID,name))
	else
		self:Print(L["There is no enchant on the weapon or enchant unknown."])
	end
end

--returns information about the enchant on currently preview weapon
function GlowFoSho:GetCurrEnchant()
	local enchant = enchants[currWeaponEnchant]
	if enchant then
		return enchant.enchantID, currWeaponEnchant, enchant.formulaID, enchant.glow
	end
end

--[[

Data

--]]

--enchant list
enchants = {
	["Agility (2H)"] = {
		enchantID = "2646",
		formulaID = "27837",
		glow = true,
	},
	["Agility"] = {
		enchantID = "2564",
		formulaID = "23800",
		glow = true,
	},
	["Battlemaster"] = {
		enchantID = "2675",
		formulaID = "28004",
		glow = true,
	},
	["Crusader"] = {
		enchantID = "1900",
		formulaID = "20034",
		glow = true,
	},
	["Demonslaying"] = {
		enchantID = "912",
		formulaID = "13915",
		glow = true,
	},
	["Fiery Weapon"] = {
		enchantID = "803",
		formulaID = "13898",
		glow = true,
	},
	["Greater Impact (2H)"] = {
		enchantID = "963",
		formulaID = "13937",
		glow = true,
	},
	["Greater Striking"] = {
		enchantID = "805",
		formulaID = "13943",
		glow = true,
	},
	["Healing Power"] = {
		enchantID = "2505",
		formulaID = "22750",
		glow = true,
	},
	["Icy Chill"] = {
		enchantID = "1894",
		formulaID = "20029",
		glow = true,
	},
	["Impact (2H)"] = {
		enchantID = "1897",
		formulaID = "13695",
		glow = true,
	},
	["Lesser Beastslayer"] = {
		enchantID = "853",
		formulaID = "13653",
		glow = true,
	},
	["Lesser Elemental Slayer"] = {
		enchantID = "854",
		formulaID = "13655",
		glow = true,
	},
	["Lesser Impact (2H)"] = {
		enchantID = "943",
		formulaID = "13529",
		glow = true,
	},
	["Lesser Intellect (2H)"] = {
		enchantID = "723",
		formulaID = "7793",
		glow = false,
	},
	["Lesser Spirit (2H)"] = {
		enchantID = "255",
		formulaID = "13380",
		glow = false,
	},
	["Lesser Striking"] = {
		enchantID = "241",
		formulaID = "13503",
		glow = false,
	},
	["Lifestealing"] = {
		enchantID = "1898",
		formulaID = "20032",
		glow = true,
	},
	["Major Agility (2H)"] = {
		enchantID = "2670",
		formulaID = "27977",
		glow = true,
	},
	["Major Healing"] = {
		enchantID = "2343",
		formulaID = "34010",
		glow = true,
	},
	["Major Intellect (2H)"] = {
		enchantID = "1904",
		formulaID = "20036",
		glow = true,
	},
	["Major Intellect"] = {
		enchantID = "2666",
		formulaID = "27968",
		glow = true,
	},
	["Major Spellpower"] = {
		enchantID = "2669",
		formulaID = "27975",
		glow = true,
	},
	["Major Spirit (2H)"] = {
		enchantID = "1903",
		formulaID = "20035",
		glow = true,
	},
	["Major Striking"] = {
		enchantID = "963",
		formulaID = "27967",
		glow = true,
	},
	["Mighty Intellect"] = {
		enchantID = "2568",
		formulaID = "23804",
		glow = true,
	},
	["Mighty Spirit"] = {
		enchantID = "2567",
		formulaID = "23803",
		glow = true,
	},
	["Minor Beastslayer"] = {
		enchantID = "249",
		formulaID = "7786",
		glow = true,
	},
	["Minor Impact (2H)"] = {
		enchantID = "241",
		formulaID = "7745",
		glow = false,
	},
	["Minor Striking"] = {
		enchantID = "250",
		formulaID = "7788",
		glow = false,
	},
	["Mongoose"] = {
		enchantID = "2673",
		formulaID = "27984",
		glow = true,
	},
	["Potency"] = {
		enchantID = "2668",
		formulaID = "27972",
		glow = true,
	},
	["Savagery (2H)"] = {
		enchantID = "2667",
		formulaID = "27971",
		glow = true,
	},
	["Soulfrost"] = {
		enchantID = "2672",
		formulaID = "27982",
		glow = true,
	},
	["Spell Power"] = {
		enchantID = "2504",
		formulaID = "22749",
		glow = true,
	},
	["Spellsurge"] = {
		enchantID = "2674",
		formulaID = "28003",
		glow = true,
	},
	["Strength"] = {
		enchantID = "2563",
		formulaID = "23799",
		glow = true,
	},
	["Striking"] = {
		enchantID = "943",
		formulaID = "13693",
		glow = true,
	},
	["Sunfire"] = {
		enchantID = "2671",
		formulaID = "27981",
		glow = true,
	},
	["Superior Impact (2H)"] = {
		enchantID = "1896",
		formulaID = "20030",
		glow = true,
	},
	["Superior Striking"] = {
		enchantID = "1897",
		formulaID = "20031",
		glow = true,
	},
	["Unholy Weapon"] = {
		enchantID = "1899",
		formulaID = "20033",
		glow = true,
	},
	["Winter's Might"] = {
		enchantID = "2443",
		formulaID = "21931",
		glow = false,
	},
}